@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de
- * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'26 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * CPIFace text modes master mode and window handler
  *
@@ -126,19 +126,21 @@ static void cpiSetFocus (struct cpifaceSessionAPI_t *cpifaceSession, const char
 	cpiFocus=mode;
 	mode->active=1;
 	strcpy(cpiFocusHandle, cpiFocus->handle);
-	cpiTextRecalc (&cpifaceSessionAPI.Public);
 }
 
 void cpiTextSetMode (struct cpifaceSessionAPI_t *cpifaceSession, const char *name)
 {
-	if (!name)
-		name=cpiFocusHandle;
 	if (!modeactive)
 	{
-		strcpy(cpiFocusHandle, name);
+		if (name && (cpiFocusHandle != name))
+		{
+			strcpy(cpiFocusHandle, name);
+		}
 		cpiSetMode("text");
-	} else
-		cpiSetFocus (cpifaceSession, name);
+	} else {
+		cpiSetFocus (cpifaceSession, name ? name : cpiFocusHandle);
+		cpiTextRecalc (&cpifaceSessionAPI.Public);
+	}
 }
 
 void cpiTextRecalc (struct cpifaceSessionAPI_t *cpifaceSession)
@@ -435,16 +437,21 @@ void cpiTextRecalc (struct cpifaceSessionAPI_t *cpifaceSession)
 #endif
 }
 
-static void txtSetMode (struct cpifaceSessionAPI_t *cpifaceSession)
+static int txtSetMode (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	struct cpitextmoderegstruct *mode;
 	plSetTextMode(fsScrType);
 	fsScrType=plScrType;
+
 	for (mode=cpiTextActModes; mode; mode=mode->nextact)
 	{
 		mode->Event (cpifaceSession, cpievSetMode);
 	}
+
+	cpiSetFocus (cpifaceSession, cpiFocusHandle);
 	cpiTextRecalc (&cpifaceSessionAPI.Public);
+
+	return 0;
 }
 
 static void txtDraw (struct cpifaceSessionAPI_t *cpifaceSession)
@@ -628,7 +635,6 @@ static int txtOpenMode (struct cpifaceSessionAPI_t *cpifaceSession)
 		mode->nextact=cpiTextActModes;
 		cpiTextActModes=mode;
 	}
-	cpiSetFocus (cpifaceSession, cpiFocusHandle);
 
 	return 1;
 }
