@@ -1,3 +1,23 @@
+/* OpenCP Module Player
+ * copyright (c) 2024-'26 Stian Skjelstad <stian.skjelstad@gmail.com>
+ *
+ * Support for accessing https://modland.com from the filebrowser
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
+
 #include "config.h"
 #include <errno.h>
 #include <stdio.h>
@@ -23,6 +43,10 @@
 #include "stuff/file.h"
 #include "stuff/framelock.h"
 #include "stuff/poutput.h"
+#ifdef _WIN32
+# include <stuff/utf-16.h>
+#endif
+
 
 struct modland_com_fileentry_t
 {
@@ -702,4 +726,4 @@ static void modland_com_done (struct PluginCloseAPI_t *API)
 	modland_com.mirrorcustom = 0;
 }
 
-DLLEXTINFO_CORE_PREFIX struct linkinfostruct dllextinfo = {.name = "modland-com", .desc = "OpenCP virtual modland.com filebrowser (c) 2024 Stian Skjelstad", .ver = DLLVERSION, .sortindex = 60, .PluginInit = modland_com_init, .PluginClose = modland_com_done};
+DLLEXTINFO_CORE_PREFIX struct linkinfostruct dllextinfo = {.name = "modland-com", .desc = "OpenCP virtual modland.com filebrowser (c) 2024-'26 Stian Skjelstad", .ver = DLLVERSION, .sortindex = 60, .PluginInit = modland_com_init, .PluginClose = modland_com_done};
