@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2019-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2019-'26 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * HVLPlay note dots routines
  *
@@ -25,7 +25,7 @@
 #include "hvlpdots.h"
 #include "player.h"
 
-int __attribute__ ((visibility ("internal"))) hvlGetDots (struct cpifaceSessionAPI_t *cpifaceSession, struct notedotsdata *d, int max)
+OCP_INTERNAL int hvlGetDots (struct cpifaceSessionAPI_t *cpifaceSession, struct notedotsdata *d, int max)
 {
 	int pos=0;
 	int i;
