@@ -61,11 +61,11 @@ CmodPlayer::~CmodPlayer()
 void CmodPlayer::gettrackdata(unsigned char pattern, void (*callback)(void *arg, unsigned char row, unsigned char channel, unsigned char note, TrackedCmds command, unsigned char inst, unsigned char volume, unsigned char param), void *arg)
 {
   if (pattern >= npats) return; //throw std::invalid_argument( "pattern out of range");
-  for (int channel = 0; channel < nchans; channel++) {
+  for (unsigned int channel = 0; channel < nchans; channel++) {
     unsigned short track = trackord[pattern][channel];
     if (!track) continue;
     track--;
-    for (int row = 0; row < nrows; row++) {
+    for (unsigned int row = 0; row < nrows; row++) {
       unsigned char note = tracks[track][row].note;
       TrackedCmds command=TrackedCmdNone;
       unsigned char inst = tracks[track][row].inst;
