@@ -146,14 +146,14 @@ void c64::setBaseSid(c64sid *s)
 bool c64::addExtraSid(c64sid *s, int address)
 {
     // Check for valid address in the IO area range ($dxxx)
-    if ((address & 0xf000) != 0xd000)
+    if ((address & 0xf000) != 0xd000) UNLIKELY
         return false;
 
     const int idx = (address >> 8) & 0xf;
 
     // Only allow second SID chip in SID area ($d400-$d7ff)
     // or IO Area ($de00-$dfff)
-    if ((idx < 0x4) || ((idx > 0x7) && (idx < 0xe)))
+    if ((idx < 0x4) || ((idx > 0x7) && (idx < 0xe))) UNLIKELY
         return false;
 
     // Add new SID bank
@@ -174,7 +174,7 @@ bool c64::addExtraSid(c64sid *s, int address)
     return true;
 }
 
-c64::~c64()
+void c64::deleteSids(sidBankMap_t &extraSidBanks)
 {
     for (auto sidBank: extraSidBanks)
         delete sidBank.second;
@@ -182,16 +182,18 @@ c64::~c64()
     extraSidBanks.clear();
 }
 
+c64::~c64()
+{
+    deleteSids(extraSidBanks);
+}
+
 void c64::clearSids()
 {
     sidBank.setSID(nullptr);
 
     resetIoBank();
 
-    for (auto sidBank: extraSidBanks)
-        delete sidBank.second;
-
-    extraSidBanks.clear();
+    deleteSids(extraSidBanks);
 }
 
 }
