@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2005-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2005-'26 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Timidity glue logic
  *
@@ -30,6 +30,9 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include "types.h"
+#ifdef _WIN32
+# include <sysinfoapi.h>
+#endif
 #include "boot/psetting.h"
 #include "cpikaraoke.h"
 #include "cpiface/cpiface.h"
@@ -459,17 +462,17 @@ static int read_user_config_file(struct cpifaceSessionAPI_t *cpifaceSession)
     int status;
 
 /* timidity.cfg or _timidity.cfg or .timidity.cfg*/
-    snprintf(path, sizeof (path), "%s" PATH_STRING "timidity.cfg", cpifaceSession->configAPI->HomePath);
+    snprintf(path, sizeof (path), "%s" "timidity.cfg", cpifaceSession->configAPI->HomePath); /* Vanilla TiMidity only scans this on WIN32 */
     status = read_config_file(&tc, path, 0, 1);
     if (status != 3 /* READ_CONFIG_FILE_NOT_FOUND */ )
         return status;
 
-    snprintf(path, sizeof (path), "%s" PATH_STRING "_timidity.cfg", cpifaceSession->configAPI->HomePath);
+    snprintf(path, sizeof (path), "%s" "_timidity.cfg", cpifaceSession->configAPI->HomePath); /* Vanilla TiMidity only scans this on WIN32 */
     status = read_config_file(&tc, path, 0, 1);
     if (status != 3 /* READ_CONFIG_FILE_NOT_FOUND */)
         return status;
 
-    snprintf(path, sizeof (path), "%s" PATH_STRING ".timidity.cfg", cpifaceSession->configAPI->HomePath);
+    snprintf(path, sizeof (path), "%s" ".timidity.cfg", cpifaceSession->configAPI->HomePath);
     status = read_config_file(&tc, path, 0, 1);
     if (status != 3 /*READ_CONFIG_FILE_NOT_FOUND*/)
         return status;
@@ -966,6 +969,64 @@ static int emulate_main_start(struct timiditycontext_t *c, struct cpifaceSession
 	}*/
 	if (!c->got_a_configuration)
 	{
+#ifdef _WIN32
+		DWORD length;
+		uint16_t *wpath;
+		char *path, *filepath;
+		if (!(length = GetWindowsDirectoryW (NULL, 0)))
+		{
+			goto other_global;
+		}
+		if (!(wpath = calloc (length, sizeof (uint16_t))))
+		{
+			goto other_global;
+		}
+		if (GetWindowsDirectoryW (wpath, length) != (length-1))
+		{
+			free (wpath);
+			goto other_global;
+		}
+		path = cpifaceSession->utf16_to_utf8 (wpath);
+		free (wpath); wpath = 0;
+		if (!path)
+		{
+			goto other_global;
+		}
+		length = strlen (path) + 13 + 1;
+		filepath = malloc (length);
+		if (!filepath)
+		{
+			free (path);
+			goto other_global;
+		}
+		snprintf (filepath, length, "%s\\TIMIDITY.CFG", path);
+		free (path);
+		if (!read_config_file(&tc, filepath, 0, 0)) // C:\WINDOWS\timidity.cfg
+		{
+			c->got_a_configuration = 1;
+		}
+		free (filepath);
+
+other_global:
+		if (!c->got_a_configuration)
+		{
+			if (!read_config_file (&tc, "C:\\timidity\\timidity.cfg", 0, 0))
+			{
+				c->got_a_configuration = 1;
+			}
+		}
+		if (!c->got_a_configuration)
+		{
+			if (!read_config_file (&tc, "C:\\TiMidity++\\timidity.cfg", 0, 0))
+			{
+				c->got_a_configuration = 1;
+			}
+		}
+		if (!c->got_a_configuration)
+		{
+			cpifaceSession->cpiDebug (cpifaceSession, "[TiMidity++ MID] Warning: Unable to find global timidity.cfg file\n");
+		}
+#else
 		/* test CONFIG_FILE first if it is defined to be something else than one of our standard paths */
 		if (strcmp(CONFIG_FILE, "/etc/timidity/timidity.cfg") &&
 		    strcmp(CONFIG_FILE, "/etc/timidity.cfg") &&
@@ -984,11 +1045,11 @@ static int emulate_main_start(struct timiditycontext_t *c, struct cpifaceSession
 			c->got_a_configuration = 1;
 		} else if (!read_config_file(&tc, "/usr/share/timidity/timidity.cfg", 0, 0))
 		{
-		c->got_a_configuration = 1;
+			c->got_a_configuration = 1;
 		} else {
 			cpifaceSession->cpiDebug (cpifaceSession, "[TiMidity++ MID] Warning: Unable to find global timidity.cfg file\n");
 		}
-
+#endif
 		if (read_user_config_file(cpifaceSession))
 		{
 			cpifaceSession->cpiDebug (cpifaceSession, "[TiMidity++ MID] Error: Syntax error in ~/.timidity.cfg\n");
@@ -1635,6 +1696,14 @@ play_reload: /* Come here to reload MIDI file */
 
 play_end:
 	free_all_midi_file_info (&tc);
+#if defined(TILD_SCHEME_ENABLE)
+        free (tc.url_expand_home_dir_path);
+	free (tc.url_unexpand_home_dir_path);
+        tc.url_expand_home_dir_path = 0;
+        tc.url_unexpand_home_dir_path = 0;
+#endif
+	free (tc.current_filename);
+	tc.current_filename = 0;
 
 	if(wrdt->opened)
 		wrdt->end();
@@ -2017,8 +2086,29 @@ static void doTimidityClosePlayer(struct cpifaceSessionAPI_t *cpifaceSession, in
 	}
 
 	free_all_midi_file_info (&tc);
+#if defined(TILD_SCHEME_ENABLE)
+        free (tc.url_expand_home_dir_path);
+        free (tc.url_unexpand_home_dir_path);
+        tc.url_expand_home_dir_path = 0;
+        tc.url_unexpand_home_dir_path = 0;
+#endif
+	free (tc.current_filename);
+	tc.current_filename = 0;
 }
 
+#ifdef _WIN32
+static uint16_t *timidity_utf8_to_utf16_LFN (struct timiditycontext_t *c, const char *src)
+{
+	return ((struct cpifaceSessionAPI_t *)(c->contextowner))->utf8_to_utf16_LFN (src, 0);
+}
+
+static char *timidity_utf16_to_utf8 (struct timiditycontext_t *c, const uint16_t *src)
+{
+	return ((struct cpifaceSessionAPI_t *)(c->contextowner))->utf16_to_utf8 (src);
+}
+
+#endif
+
 OCP_INTERNAL int timidityOpenPlayer (const char *path, uint8_t *buffer, size_t bufferlen, struct ocpfilehandle_t *file, struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	uint32_t gmibuflen;
@@ -2131,6 +2221,10 @@ OCP_INTERNAL int timidityOpenPlayer (const char *path, uint8_t *buffer, size_t b
 	tc.ctl_timestamp_last_secs = -1;
 	tc.ctl_timestamp_last_voices = -1;
 	tc.play_midi_file_last_rc = RC_NONE;
+#ifdef _WIN32
+	tc.utf8_to_utf16_LFN = timidity_utf8_to_utf16_LFN;
+	tc.utf16_to_utf8 = timidity_utf16_to_utf8;
+#endif
 
 	if (!cpifaceSession->plrDevAPI)
 	{
