@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2009-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2009-'26 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * SDL graphic driver
  *
@@ -163,7 +163,9 @@ again:
 	} else {
 		current_surface = SDL_SetVideoMode(width, height, 0, SDL_ANYFORMAT|SDL_RESIZABLE|SDL_HWSURFACE);
 		if (!current_surface)
+		{
 			current_surface = SDL_SetVideoMode(width, height, 0, SDL_ANYFORMAT|SDL_RESIZABLE|SDL_SWSURFACE);
+		}
 	}
 	Console.TextGUIOverlay = current_surface->format->BytesPerPixel != 1;
 
@@ -171,6 +173,9 @@ again:
 	{
 		switch (Console.CurrentFont)
 		{
+			case _16x32:
+				Console.CurrentFont = _8x16;
+				break;
 			case _8x16:
 				Console.CurrentFont = _8x8;
 				break;
@@ -204,11 +209,27 @@ again:
 		exit(-1);
 	}
 
+	do { /* resolve Console.CurrentMode */
+		int i;
+		for (i=0; i < 8; i++)
+		{
+			if ((Console.TextWidth   == mode_tui_data[i].text_width) &&
+			    (Console.TextHeight  == mode_tui_data[i].text_height) &&
+			    (Console.CurrentFont == mode_tui_data[i].font))
+			{
+				break;
+			}
+		}
+		Console.LastTextMode = Console.CurrentMode = i;
+	} while (0);
+
 	sdl_gFlushPal ();
 
 	___push_key(VIRT_KEY_RESIZE);
 }
 
+static int cachemode = -1;
+
 static void sdl_SetTextMode (unsigned char x)
 {
 	set_state = set_state_textmode;
@@ -219,25 +240,16 @@ static void sdl_SetTextMode (unsigned char x)
 		return;
 	}
 
-	if (x==255)
+	Console.CurrentFont = sdl_CurrentFontWanted;
+
+	if (cachemode >= 0)
 	{
 		sdl_SetGraphMode (-1);
-
-		if (current_surface)
-		{
-			/* This surface belongs to SDL internals */
-			/* SDL_FreeSurface(current_surface); */
-			current_surface = 0;
-		}
-		Console.CurrentMode = 255;
-		return; /* gdb helper */
 	}
 
 	/* if invalid mode, set it to custom */
 	if (x>7)
 	{
-		x=8;
-
 		set_state_textmode(
 			do_fullscreen,
 			last_text_width,
@@ -250,12 +262,8 @@ static void sdl_SetTextMode (unsigned char x)
 			mode_gui_data[mode_tui_data[x].gui_mode].width,
 			mode_gui_data[mode_tui_data[x].gui_mode].height);
 	}
-
-	Console.LastTextMode = Console.CurrentMode = x;
 }
 
-static int cachemode = -1;
-
 static void set_state_graphmode(int fullscreen, int width, int height)
 {
 	mode_gui_t mode;
@@ -295,9 +303,10 @@ static void set_state_graphmode(int fullscreen, int width, int height)
 
 	if ((do_fullscreen = fullscreen))
 	{
-
 		if (fullscreen_info[mode].is_possible)
+		{
 			current_surface = SDL_SetVideoMode(fullscreen_info[mode].resolution.w, fullscreen_info[mode].resolution.h, 0, fullscreen_info[mode].flags | SDL_ANYFORMAT);
+		}
 	}
 
 	if (!current_surface)
@@ -305,7 +314,9 @@ static void set_state_graphmode(int fullscreen, int width, int height)
 		do_fullscreen = 0;
 		current_surface = SDL_SetVideoMode(width, height, 0, SDL_ANYFORMAT|SDL_HWSURFACE);
 		if (!current_surface)
+		{
 			current_surface = SDL_SetVideoMode(width, height, 0, SDL_ANYFORMAT|SDL_SWSURFACE);
+		}
 	}
 	Console.TextGUIOverlay = current_surface->format->BytesPerPixel != 1;
 
@@ -313,6 +324,7 @@ static void set_state_graphmode(int fullscreen, int width, int height)
 	Console.TextHeight        = height/16;
 	Console.GraphBytesPerLine = width;
 	Console.GraphLines        = height;
+	Console.CurrentFont       = _8x16;
 
 	plScrRowBytes = Console.TextWidth * 2;
 
@@ -358,9 +370,9 @@ quick:
 	return 0;
 }
 
-static void sdl_vga13 (void)
+static int sdl_vga13 (void)
 {
-	sdl_SetGraphMode (13);
+	return sdl_SetGraphMode (13);
 }
 
 static void FindFullscreenModes_SDL(Uint32 flags)
@@ -438,6 +450,8 @@ static void sdl_DisplaySetupTextMode(void)
 		swtext_displaystr_cp437(1, 0, 0x07, "1:  font-size:", 14);
 		swtext_displaystr_cp437(1, 15, Console.CurrentFont == _8x8 ? 0x0f : 0x07, "8x8", 3);
 		swtext_displaystr_cp437(1, 19, Console.CurrentFont == _8x16 ? 0x0f : 0x07, "8x16", 4);
+		swtext_displaystr_cp437(1, 24, Console.CurrentFont == _16x32 ? 0x0f : 0x07, "16x32", 5);
+
 /*
 		swtext_displaystr_cp437(2, 0, 0x07, "2:  fullscreen: ", 16);
 		swtext_displaystr_cp437(3, 0, 0x07, "3:  resolution in fullscreen:", 29);*/
@@ -454,7 +468,7 @@ static void sdl_DisplaySetupTextMode(void)
 		{
 			case '1':
 				/* we can assume that we are in text-mode if we are here */
-				sdl_CurrentFontWanted = Console.CurrentFont = (Console.CurrentFont == _8x8)?_8x16:_8x8;
+				sdl_CurrentFontWanted = Console.CurrentFont = (Console.CurrentFont == _8x8) ? _8x16 : (Console.CurrentFont == _8x16) ? _16x32 : _8x8;
 				set_state_textmode(do_fullscreen, Console.GraphBytesPerLine, Console.GraphLines);
 				cfSetProfileInt(cfScreenSec, "fontsize", Console.CurrentFont, 10);
 				break;
@@ -468,7 +482,7 @@ static const char *sdl_GetDisplayTextModeName(void)
 {
 	static char mode[48];
 	snprintf(mode, sizeof(mode), "res(%dx%d), font(%s)%s", Console.TextWidth, Console.TextHeight,
-		Console.CurrentFont == _8x8 ? "8x8" : "8x16", do_fullscreen?" fullscreen":"");
+		Console.CurrentFont == _8x8 ? "8x8" : Console.CurrentFont == _8x16 ? "8x16" : "16x32", do_fullscreen?" fullscreen":"");
 	return mode;
 }
 
@@ -1078,6 +1092,9 @@ static void RefreshScreenGraph(void)
 	} else if (Console.CurrentFont == _8x16)
 	{
 		fontengine_8x16_iterate ();
+	} else if (Console.CurrentFont == _16x32)
+	{
+		fontengine_16x32_iterate ();
 	}
 }
 
@@ -1122,7 +1139,7 @@ static uint32_t utf16_try_decode (uint16_t *data_he, int *length)
 		return 0;
 	}
 	(*length) = 0;
-	return ((uint_fast32_t)(data_he[0] & 0x03ff) << 10) | (data_he[1] & 0x03ff);
+	return (((uint_fast32_t)(data_he[0] & 0x03ff) << 10) | (data_he[1] & 0x03ff)) + 0x10000;
 }
 
 /* duplicate from utf8.c: poutput.so is not available yet, and SDL 1.x is only for legacy use */
@@ -1467,7 +1484,7 @@ int sdl_init(void)
 	sdl_CurrentFontWanted = Console.CurrentFont = cfGetProfileInt(cfScreenSec, "fontsize", _8x16, 10);
 	if (Console.CurrentFont > _FONT_MAX)
 	{
-		Console.CurrentFont = _8x16;
+		Console.CurrentFont = _16x32;
 	}
 	last_text_width  = Console.GraphBytesPerLine = saturate(cfGetProfileInt(cfScreenSec, "winwidth",  1280, 10), 640, 16384);
 	last_text_height = Console.GraphLines        = saturate(cfGetProfileInt(cfScreenSec, "winheight", 1024, 10), 400, 16384);
