@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2005-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2005-'26 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * X11 graphic driver
  *
@@ -430,6 +430,11 @@ static void WindowResized_Textmode(unsigned int width, unsigned int height)
 
 	Console.CurrentFont = x11_CurrentFontWanted;
 
+	if (Console.CurrentFont >= _16x32)
+	{
+		if ( ( Console.GraphBytesPerLine < ( 16 * 80 ) ) || ( Console.GraphLines < ( 32 * 25 ) ) )
+			Console.CurrentFont = _8x16;
+	}
 	if (Console.CurrentFont >= _8x16)
 	{
 		if ( ( Console.GraphBytesPerLine < ( 8 * 80 ) ) || ( Console.GraphLines < ( 16 * 25 ) ) )
@@ -447,17 +452,22 @@ static void WindowResized_Textmode(unsigned int width, unsigned int height)
 			Console.TextWidth = Console.GraphBytesPerLine / 8;
 			Console.TextHeight = Console.GraphLines / 16;
 			break;
+		case _16x32:
+			Console.TextWidth = Console.GraphBytesPerLine / 16;
+			Console.TextHeight = Console.GraphLines / 32;
+			break;
 	}
 	plScrRowBytes = Console.TextWidth * 2;
 
 	destroy_image();
 	create_image();
+
 	if (virtual_framebuffer)
 	{
 		free(virtual_framebuffer);
 		virtual_framebuffer=0;
 	}
-	if ((x11_depth!=8) || (Console.GraphBytesPerLine != image->bytes_per_line))
+	if ((x11_depth != 8) || (Console.GraphBytesPerLine != image->bytes_per_line))
 	{
 		virtual_framebuffer = malloc (Console.GraphBytesPerLine * Console.GraphLines);
 		Console.VidMem = virtual_framebuffer;
@@ -467,14 +477,23 @@ static void WindowResized_Textmode(unsigned int width, unsigned int height)
 	}
 	memset (Console.VidMem, 0, Console.GraphBytesPerLine * Console.GraphLines);
 
+	do { /* resolve Console.CurrentMode */
+		int i;
+		for (i=0; i < 8; i++)
+		{
+			if ((Console.TextWidth   == modes[i].charx) &&
+			    (Console.TextHeight  == modes[i].chary) &&
+			    (Console.CurrentFont == modes[i].bigfont))
+			{
+				break;
+			}
+		}
+		Console.LastTextMode = Console.CurrentMode = i;
+	} while (0);
+
+
 	if (!do_fullscreen)
 	{
-		if ((Console.CurrentMode != 8) &&
-		    ((modes[Console.CurrentMode].windowy != height) ||
-		     (modes[Console.CurrentMode].windowx != width)))
-		{
-			Console.LastTextMode = Console.CurrentMode = 8;
-		}
 		Textmode_Window_Height = height;
 		Textmode_Window_Width = width;
 	}
@@ -517,16 +536,19 @@ static void set_state_graphmode(int FullScreen)
 		SizeHints.min_height = SizeHints.max_height = Console.GraphLines;
 	}
 
+	//fprintf (stderr, "[x11] XSetWMNormalHints (min_width=%d max_width=%d min_height=%d max_height=%d\n", SizeHints.min_width, SizeHints.max_width, SizeHints.min_height, SizeHints.max_height);
 	XSetWMNormalHints(mDisplay, window, &SizeHints);
 	motif_decoration(window, !do_fullscreen);
 	ewmh_fullscreen(window, do_fullscreen);
 
 	if (do_fullscreen)
 	{
+		//fprintf (stderr, "[x11] /* do_fullscreen */ XResizeWindow (width=%d height=%d)\n", Graphmode_modeline->hdisplay, Graphmode_modeline->vdisplay);
 		XResizeWindow(mDisplay, window, Graphmode_modeline->hdisplay, Graphmode_modeline->vdisplay);
 		XSync(mDisplay, FALSE);
 		//XClearWindow(mDisplay, window);
 	} else {
+		//fprintf (stderr, "[x11] XResizeWindow (width=%d height=%d)\n", Console.GraphBytesPerLine, Console.GraphLines);
 		XResizeWindow(mDisplay, window, Console.GraphBytesPerLine, Console.GraphLines);
 		XSync(mDisplay, FALSE);
 	}
@@ -603,6 +625,12 @@ static void TextModeSetState(FontSizeEnum FontSize, int FullScreen)
 	switch (FontSize)
 	{
 		/* find the min value, if screen is too small, force smaller font */
+		case _16x32:
+			Textmode_SizeHints.min_width = 16 * 80;
+			Textmode_SizeHints.min_height = 32 * 25;
+			if ( (Textmode_SizeHints.min_width <= Textmode_SizeHints.max_width) && (Textmode_SizeHints.min_height <= Textmode_SizeHints.max_height) )
+				break;
+			FontSize = _8x16; /* drop through */
 		case _8x16:
 			Textmode_SizeHints.min_width = 8 * 80;
 			Textmode_SizeHints.min_height = 16 * 25;
@@ -645,7 +673,6 @@ static void TextModeSetState(FontSizeEnum FontSize, int FullScreen)
 		Console.GraphLines = Textmode_Window_Height;
 		Console.GraphBytesPerLine = Textmode_Window_Width;
 	}
-	___push_key(VIRT_KEY_RESIZE);
 
 	if (xvidmode_event_base >= 0)
 	{
@@ -685,6 +712,10 @@ static void TextModeSetState(FontSizeEnum FontSize, int FullScreen)
 			Console.TextWidth = Console.GraphBytesPerLine / 8;
 			Console.TextHeight = Console.GraphLines / 16;
 			break;
+		case _16x32:
+			Console.TextWidth = Console.GraphBytesPerLine / 16;
+			Console.TextHeight = Console.GraphLines / 32;
+			break;
 	}
 	plScrRowBytes = Console.TextWidth * 2;
 
@@ -705,6 +736,22 @@ static void TextModeSetState(FontSizeEnum FontSize, int FullScreen)
 		Console.VidMem = (uint8_t *)image->data;
 	}
 	memset (Console.VidMem, 0, Console.GraphBytesPerLine * Console.GraphLines);
+
+	do { /* resolve Console.CurrentMode */
+		int i;
+		for (i=0; i < 8; i++)
+		{
+			if ((Console.TextWidth   == modes[i].charx) &&
+			    (Console.TextHeight  == modes[i].chary) &&
+			    (Console.CurrentFont == modes[i].bigfont))
+			{
+				break;
+			}
+		}
+		Console.LastTextMode = Console.CurrentMode = i;
+	} while (0);
+
+	___push_key(VIRT_KEY_RESIZE);
 }
 
 static void x11_common_event_loop(void)
@@ -804,7 +851,7 @@ static void x11_common_event_loop(void)
 				}
 				break;
 			case ConfigureNotify:
-				//fprintf(stderr, "configure notify\n");
+				//fprintf(stderr, "[X11] configure notify\n");
 				if ((Console.GraphBytesPerLine == event.xconfigure.width) && (Console.GraphLines == event.xconfigure.height))
 				{
 					/* event was probably only a move */
@@ -833,6 +880,10 @@ static void x11_common_event_loop(void)
 							x &= ~7;
 							y &= ~15;
 							break;
+						case _16x32:
+							x &= ~15;
+							y &= ~31;
+							break;
 					}
 					fprintf(stderr, "TEREWRWR\n");
 					//XResizeWindow(mDisplay, window, x, y);
@@ -1067,6 +1118,7 @@ static void create_window(void)
 	xswa.border_pixel=WhitePixel(mDisplay, mScreen);
 	xswa.event_mask=VisibilityChangeMask|FocusChangeMask|ExposureMask|KeyPressMask|KeyReleaseMask|ButtonPressMask|ButtonReleaseMask/*|ResizeRedirectMask*/|StructureNotifyMask;
 	xswa.override_redirect = False;
+	//fprintf (stderr, "[x11] XCreateWindow(width=%d height=%d)\n", Console.GraphBytesPerLine, Console.GraphLines);
 	if (!(window = XCreateWindow (mDisplay,
 	                              DefaultRootWindow(mDisplay),
 	                              0, 0,
@@ -1137,7 +1189,7 @@ static void create_image(void)
 	{
 		if (image)
 		{
-			fprintf (stderr, "image already set, memory will be lost\n");
+			fprintf (stderr, "[x11] image already set, memory will be lost\n");
 		}
 		shm_completiontype = XShmGetEventBase(mDisplay) + ShmCompletion;
 		if (!(image = XShmCreateImage (mDisplay,
@@ -1214,7 +1266,7 @@ static void create_image(void)
 		data = calloc (Console.GraphBytesPerLine, bytes_per_line);
 		if (!data)
 		{
-			fprintf (stderr, "calloc() before XCreateImage() failed\n");
+			fprintf (stderr, "[x11] calloc() before XCreateImage() failed\n");
 			exit(-1);
 		}
 		if (!(image = XCreateImage (
@@ -1351,6 +1403,7 @@ static int x11_SetGraphMode (int high)
 		Console.GraphLines = 480;
 		Graphmode_modeline = modelines[MODE_640x480];
 	}
+	Console.CurrentFont = _8x16;
 	if (!Graphmode_modeline)
 	{
 		fprintf(stderr, "[x11] unable to find modeline, this should not happen\n");
@@ -1385,9 +1438,9 @@ quick:
 	return 0;
 }
 
-static void x11_vga13(void)
+static int x11_vga13(void)
 {
-	x11_SetGraphMode (13);
+	return x11_SetGraphMode (13);
 }
 
 static void x11_SetTextMode (unsigned char x)
@@ -1409,32 +1462,6 @@ static void x11_SetTextMode (unsigned char x)
 		destroy_image();
 	}
 
-	if (x==255)
-	{
-		destroy_image();
-
-		if (window)
-		{
-			vo_showcursor(mDisplay, window);
-			if (we_have_fullscreen)
-				ewmh_fullscreen(window, 0);
-			XDestroyWindow(mDisplay, window);
-			window=(Window)0;
-		}
-		if (xvidmode_event_base>=0)
-		{
-			XF86VidModeSwitchToMode(mDisplay, mScreen, &default_modeline);
-		}
-
-		XUngrabKeyboard(mDisplay, CurrentTime);
-		XUngrabPointer(mDisplay, CurrentTime);
-
-		XSync(mDisplay, False);
-		Console.CurrentMode = 255;
-
-		return; /* gdb helper */
-	}
-
 #if 0 // We have disabled textmode from being able to change active resolution of physical display - Modern systems do not change resolution away from native the native resolution
 	/* if invalid mode, set it to zero */
 	if (x>7)
@@ -1479,8 +1506,9 @@ static void x11_SetTextMode (unsigned char x)
 	if (x>=8)
 	{
 		x=8;
-		Console.TextHeight = Textmode_Window_Height / (x11_CurrentFontWanted == _8x16 ? 16 : 8);
-		Console.TextWidth = Textmode_Window_Width / 8;
+		Console.TextHeight = Textmode_Window_Height / (x11_CurrentFontWanted == _16x32 ) ? 32 : (x11_CurrentFontWanted == _8x16) ? 16 : 8;
+		Console.TextWidth = Textmode_Window_Width / (x11_CurrentFontWanted == _16x32 ? 16 : 8);
+;
 		Console.GraphBytesPerLine = Textmode_Window_Width;
 		Console.GraphLines = Textmode_Window_Height;
 	} else {
@@ -1496,15 +1524,13 @@ static void x11_SetTextMode (unsigned char x)
 	}
 	plScrRowBytes = Console.TextWidth * 2;
 #endif
-	Console.LastTextMode = Console.CurrentMode = x;
-
 	x11_depth=XDefaultDepth(mDisplay, mScreen);
 	if (!window)
 	{
 		create_window();
 	}
 
-	TextModeSetState(x11_CurrentFontWanted /* modes[x].bigfont ? _8x16 : _8x8 */, do_fullscreen);
+	TextModeSetState(x11_CurrentFontWanted, do_fullscreen);
 
 	___push_key(VIRT_KEY_RESIZE);
 
@@ -1559,7 +1585,7 @@ static void x11_TextOverlayRemove (void *handle)
 			return;
 		}
 	}
-	fprintf (stderr, "[X11] Warning: x11_TextOverlayRemove, handle %p not found\n", handle);
+	fprintf (stderr, "[x11] Warning: x11_TextOverlayRemove, handle %p not found\n", handle);
 }
 
 static void RefreshScreenGraph(void)
@@ -1891,6 +1917,9 @@ static void RefreshScreenGraph(void)
 	} else if (Console.CurrentFont == _8x16)
 	{
 		fontengine_8x16_iterate ();
+	} else if (Console.CurrentFont == _16x32)
+	{
+		fontengine_16x32_iterate ();
 	}
 }
 
@@ -1966,7 +1995,7 @@ static void x11_DosShell (void)
 		{
 			close(2);
 			if (dup(1)!=2)
-				fprintf(stderr, __FILE__ ": dup(1) != 2\n");
+				fprintf(stderr, "[x11] dup(1) != 2\n");
 		}
 		execl(shell, shell, NULL);
 		perror("execl()");
@@ -2171,7 +2200,7 @@ static int x11_HasKey (uint16_t key)
 			return 1;
 
 		default:
-			fprintf(stderr, __FILE__ ": unknown key 0x%04x\n", (int)key);
+			fprintf(stderr, "[x11] unknown key 0x%04x\n", (int)key);
 		case KEY_ALT_ENTER:
 			return 0;
 	}
@@ -2179,9 +2208,9 @@ static int x11_HasKey (uint16_t key)
 
 static const char *x11_GetDisplayTextModeName (void)
 {
-	static char mode[32];
+	static char mode[48];
 	snprintf(mode, sizeof(mode), "res(%dx%d), font(%s)%s", Console.TextWidth, Console.TextHeight,
-		x11_CurrentFontWanted == _8x8 ? "8x8" : "8x16", do_fullscreen?" fullscreen":"");
+		x11_CurrentFontWanted == _8x8 ? "8x8" : x11_CurrentFontWanted == _8x16 ? "8x16" : "16x32", do_fullscreen?" fullscreen":"");
 	return mode;
 }
 
@@ -2197,6 +2226,7 @@ static void x11_DisplaySetupTextMode (void)
 		swtext_displaystr_cp437 (1, 0, 0x07, "1:  font-size:", 14);
 		swtext_displaystr_cp437 (1, 15, Console.CurrentFont == _8x8 ? 0x0f : 0x07, "8x8", 3);
 		swtext_displaystr_cp437 (1, 19, Console.CurrentFont == _8x16 ? 0x0f : 0x07, "8x16", 4);
+		swtext_displaystr_cp437 (1, 24, Console.CurrentFont == _16x32 ? 0x0f : 0x07, "16x32", 5);
 /*
 		swtext_displaystr_cp437 (2, 0, 0x07, "2:  fullscreen: ", 16);
 		swtext_displaystr_cp437 (3, 0, 0x07, "3:  resolution in fullscreen:", 29);
@@ -2215,7 +2245,7 @@ static void x11_DisplaySetupTextMode (void)
 		{
 			case '1':
 				/* we can assume that we are in text-mode if we are here */
-				x11_CurrentFontWanted = (x11_CurrentFontWanted == _8x8)?_8x16 : _8x8;
+				x11_CurrentFontWanted = (x11_CurrentFontWanted == _8x8) ? _8x16 : (x11_CurrentFontWanted == _8x16) ? _16x32 : _8x8;
 				TextModeSetState (x11_CurrentFontWanted, do_fullscreen);
 				x11_CurrentFontWanted = Console.CurrentFont;
 				cfSetProfileInt(cfScreenSec, "fontsize", Console.CurrentFont, 10);
@@ -2290,6 +2320,7 @@ int x11_init(int use_explicit)
 		default:
 		case _8x8: x11_CurrentFontWanted = _8x8; break;
 		case _8x16: x11_CurrentFontWanted = _8x16; break;
+		case _16x32: x11_CurrentFontWanted = _16x32; break;
 	}
 
 	if (x11_connect())
